package com.sun.java.util.jar.pack;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.HashMap;
import java.util.HashSet;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.io.OutputStream;

class PackageWriter extends BandStructure
{
    Package pkg;
    OutputStream finalOut;
    Package.Version packageVersion;
    Set<ConstantPool.Entry> requiredEntries;
    Map<Attribute.Layout, int[]> backCountTable;
    int[][] attrCounts;
    int[] maxFlags;
    List<Map<Attribute.Layout, int[]>> allLayouts;
    Attribute.Layout[] attrDefsWritten;
    private Code curCode;
    private Package.Class curClass;
    private ConstantPool.Entry[] curCPMap;
    int[] codeHist;
    int[] ldcHist;
    static final /* synthetic */ boolean $assertionsDisabled;
    
    PackageWriter(Package pkg, OutputStream finalOut) throws IOException {
        this.codeHist = new int[256];
        this.ldcHist = new int[20];
        this.pkg = pkg;
        this.finalOut = finalOut;
        this.initHighestClassVersion(pkg.getHighestClassVersion());
    }
    
    void write() throws IOException {
        try {
            if (this.verbose > 0) {
                Utils.log.info("Setting up constant pool...");
            }
            this.setup();
            if (this.verbose > 0) {
                Utils.log.info("Packing...");
            }
            this.writeConstantPool();
            this.writeFiles();
            this.writeAttrDefs();
            this.writeInnerClasses();
            this.writeClassesAndByteCodes();
            this.writeAttrCounts();
            if (this.verbose > 1) {
                this.printCodeHist();
            }
            if (this.verbose > 0) {
                Utils.log.info("Coding...");
            }
            this.all_bands.chooseBandCodings();
            this.writeFileHeader();
            this.writeAllBandsTo(this.finalOut);
        }
        catch (Exception ex) {
            Utils.log.warning("Error on output: " + ex, ex);
            if (this.verbose > 0) {
                this.finalOut.close();
            }
            if ((ex instanceof IOException)) {
                throw (IOException)ex;
            }
            if ((ex instanceof RuntimeException)) {
                throw (RuntimeException)ex;
            }
            throw new Error("error packing", ex);
        }
    }
    
    void setup() {
        this.requiredEntries = new HashSet();
        this.setArchiveOptions();
        this.trimClassAttributes();
        this.collectAttributeLayouts();
        this.pkg.buildGlobalConstantPool(this.requiredEntries);
        this.setBandIndexes();
        this.makeNewAttributeBands();
        this.collectInnerClasses();
    }
    
    void chooseDefaultPackageVersion() throws IOException {
        if (this.pkg.packageVersion != null) {
            this.packageVersion = this.pkg.packageVersion;
            if (this.verbose > 0) {
                Utils.log.info("package version overridden with: " + this.packageVersion);
            }
            return;
        }
        else {
            Package.Version highestClassVersion = this.getHighestClassVersion();
            if (highestClassVersion.lessThan(Constants.JAVA6_MAX_CLASS_VERSION)) {
                this.packageVersion = Constants.JAVA5_PACKAGE_VERSION;
            }
            else if (highestClassVersion.equals(Constants.JAVA6_MAX_CLASS_VERSION) || (highestClassVersion.equals(Constants.JAVA7_MAX_CLASS_VERSION) && !this.pkg.cp.haveExtraTags())) {
                this.packageVersion = Constants.JAVA6_PACKAGE_VERSION;
            }
            else if (highestClassVersion.equals(Constants.JAVA7_MAX_CLASS_VERSION)) {
                this.packageVersion = Constants.JAVA7_PACKAGE_VERSION;
            }
            else {
                this.packageVersion = Constants.JAVA8_PACKAGE_VERSION;
            }
            if (this.verbose > 0) {
                Utils.log.info("Highest version class file: " + highestClassVersion + " package version: " + this.packageVersion);
            }
            return;
        }
    }
    
    void checkVersion() throws IOException {
        assert this.packageVersion != null;
        if (this.packageVersion.lessThan(Constants.JAVA7_PACKAGE_VERSION) && BandStructure.testBit(this.archiveOptions, 8)) {
            throw new IOException("Format bits for Java 7 must be zero in previous releases");
        }
        if (BandStructure.testBit(this.archiveOptions, -8192)) {
            throw new IOException("High archive option bits are reserved and must be zero: " + Integer.toHexString(this.archiveOptions));
        }
        return;
    }
    
    void setArchiveOptions() {
        int default_modtime = this.pkg.default_modtime;
        int default_modtime2 = this.pkg.default_modtime;
        int n = -1;
        int n2 = 0;
        this.archiveOptions |= this.pkg.default_options;
        for (Package.File file : this.pkg.files) {
            int modtime = file.modtime;
            int options = file.options;
            if (default_modtime == 0) {
                default_modtime2 = (default_modtime = modtime);
            }
            else {
                if (default_modtime > modtime) {
                    default_modtime = modtime;
                }
                if (default_modtime2 < modtime) {
                    default_modtime2 = modtime;
                }
            }
            n &= options;
            n2 |= options;
        }
        if (this.pkg.default_modtime == 0) {
            this.pkg.default_modtime = default_modtime;
        }
        if (default_modtime != 0 && default_modtime != default_modtime2) {
            this.archiveOptions |= 0x40;
        }
        if (!BandStructure.testBit(this.archiveOptions, 32) && n != -1) {
            if (BandStructure.testBit(n, 1)) {
                this.archiveOptions |= 0x20;
                --n;
                --n2;
            }
            Package pkg = this.pkg;
            pkg.default_options |= n;
            if (n != n2 || n != this.pkg.default_options) {
                this.archiveOptions |= 0x80;
            }
        }
        HashMap o = new HashMap();
        int n3 = 0;
        Package.Version java_MIN_CLASS_VERSION = null;
        Iterator<Package.Class> iterator2 = this.pkg.classes.iterator();
        while (iterator2.hasNext()) {
            Package.Version version = ((Package.Class)iterator2.next()).getVersion();
            int[] array = (int[])o.get(version);
            if (array == null) {
                array = new int[] { 0 };
                o.put(version, array);
            }
            int n4 = ++array[0];
            if (n3 < n4) {
                n3 = n4;
                java_MIN_CLASS_VERSION = version;
            }
            else {
                continue;
            }
        }
        o.clear();
        if (java_MIN_CLASS_VERSION == null) {
            java_MIN_CLASS_VERSION = Constants.JAVA_MIN_CLASS_VERSION;
        }
        this.pkg.defaultClassVersion = java_MIN_CLASS_VERSION;
        if (this.verbose > 0) {
            Utils.log.info("Consensus version number in segment is " + java_MIN_CLASS_VERSION);
        }
        if (this.verbose > 0) {
            Utils.log.info("Highest version number in segment is " + this.pkg.getHighestClassVersion());
        }
        for (Package.Class obj : this.pkg.classes) {
            if (!obj.getVersion().equals(java_MIN_CLASS_VERSION)) {
                Attribute classFileVersionAttr = this.makeClassFileVersionAttr(obj.getVersion());
                if (this.verbose > 1) {
                    Utils.log.fine("Version " + obj.getVersion() + " of " + obj + " doesn't match package version " + java_MIN_CLASS_VERSION);
                }
                obj.addAttribute(classFileVersionAttr);
            }
            else {
                continue;
            }
        }
        for (Package.File file2 : this.pkg.files) {
            long fileLength = file2.getFileLength();
            if (fileLength != (long)(int)fileLength) {
                this.archiveOptions |= 0x100;
                if (this.verbose > 0) {
                    Utils.log.info("Note: Huge resource file " + file2.getFileName() + " forces 64-bit sizing");
                    break;
                }
                break;
            }
            else {
                continue;
            }
        }
        int n5 = 0;
        int n6 = 0;
        Iterator<Package.Class> iterator5 = this.pkg.classes.iterator();
        while (iterator5.hasNext()) {
            for (Package.Class.Method method : ((Package.Class)iterator5.next()).getMethods()) {
                if (method.code != null) {
                    if (method.code.attributeSize() == 0) {
                        ++n6;
                    }
                    else if (BandStructure.shortCodeHeader(method.code) != 0) {
                        n5 += 3;
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
        }
        if (n5 > n6) {
            this.archiveOptions |= 0x4;
        }
        if (this.verbose > 0) {
            Utils.log.info("archiveOptions = 0b" + Integer.toBinaryString(this.archiveOptions));
        }
    }
    
    void writeFileHeader() throws IOException {
        this.chooseDefaultPackageVersion();
        this.writeArchiveMagic();
        this.writeArchiveHeader();
    }
    
    private void putMagicInt32(int n) throws IOException {
        for (int i = 0; i < 4; n <<= 8, ++i) {
            this.archive_magic.putByte(0xFF & n >>> 24);
        }
    }
    
    void writeArchiveMagic() throws IOException {
        this.pkg.getClass();
        this.putMagicInt32(-889270259);
    }
    
    void writeArchiveHeader() throws IOException {
        int n = 15;
        int testBit = BandStructure.testBit(this.archiveOptions, 1);
        if (testBit == 0) {
            testBit = (testBit | ((this.band_headers.length() != 0) ? 1 : 0) | ((this.attrDefsWritten.length != 0) ? 1 : 0));
            if (testBit != 0) {
                this.archiveOptions |= 0x1;
            }
        }
        if (testBit != 0) {
            n += 2;
        }
        int testBit2 = BandStructure.testBit(this.archiveOptions, 16);
        if (testBit2 == 0) {
            testBit2 = (testBit2 | ((this.archiveNextCount > 0) ? 1 : 0) | ((this.pkg.default_modtime != 0) ? 1 : 0));
            if (testBit2 != 0) {
                this.archiveOptions |= 0x10;
            }
        }
        if (testBit2 != 0) {
            n += 5;
        }
        int testBit3 = BandStructure.testBit(this.archiveOptions, 2);
        if (testBit3 == 0) {
            testBit3 |= this.pkg.cp.haveNumbers();
            if (!(testBit3 == 0)) {
                this.archiveOptions |= 0x2;
            }
        }
        if (!(testBit3 == 0)) {
            n += 4;
        }
        int testBit4 = BandStructure.testBit(this.archiveOptions, 8);
        if (testBit4 == 0) {
            testBit4 |= this.pkg.cp.haveExtraTags();
            if (!(testBit4 == 0)) {
                this.archiveOptions |= 0x8;
            }
        }
        if (!(testBit4 == 0)) {
            n += 4;
        }
        this.checkVersion();
        this.archive_header_0.putInt((int)this.packageVersion.minor);
        this.archive_header_0.putInt((int)this.packageVersion.major);
        if (this.verbose > 0) {
            Utils.log.info("Package Version for this segment:" + this.packageVersion);
        }
        this.archive_header_0.putInt(this.archiveOptions);
        assert this.archive_header_0.length() == 3;
        if (testBit2 != 0) {
            assert this.archive_header_S.length() == 0;
            this.archive_header_S.putInt(0);
            assert this.archive_header_S.length() == 1;
            this.archive_header_S.putInt(0);
            assert this.archive_header_S.length() == 2;
        }
        if (testBit2 != 0) {
            this.archive_header_1.putInt(this.archiveNextCount);
            this.archive_header_1.putInt(this.pkg.default_modtime);
            this.archive_header_1.putInt(this.pkg.files.size());
        }
        else
            assert !(!this.pkg.files.isEmpty());
        if (testBit != 0) {
            this.archive_header_1.putInt(this.band_headers.length());
            this.archive_header_1.putInt(this.attrDefsWritten.length);
        }
        else {
            assert this.band_headers.length() == 0;
            assert this.attrDefsWritten.length == 0;
        }
        this.writeConstantPoolCounts((boolean)testBit3, (boolean)testBit4);
        this.archive_header_1.putInt(this.pkg.getAllInnerClasses().size());
        this.archive_header_1.putInt((int)this.pkg.defaultClassVersion.minor);
        this.archive_header_1.putInt((int)this.pkg.defaultClassVersion.major);
        this.archive_header_1.putInt(this.pkg.classes.size());
        assert this.archive_header_0.length() + this.archive_header_S.length() + this.archive_header_1.length() == n;
        this.archiveSize0 = 0L;
        this.archiveSize1 = this.all_bands.outputSize();
        this.archiveSize0 += this.archive_magic.outputSize();
        this.archiveSize0 += this.archive_header_0.outputSize();
        this.archiveSize0 += this.archive_header_S.outputSize();
        this.archiveSize1 -= this.archiveSize0;
        if (testBit2 != 0) {
            int n2 = (int)(this.archiveSize1 >>> 32);
            int n3 = (int)(this.archiveSize1 >>> 0);
            this.archive_header_S.patchValue(0, n2);
            this.archive_header_S.patchValue(1, n3);
            int length = PackageWriter.UNSIGNED5.getLength(0);
            this.archiveSize0 += (long)(PackageWriter.UNSIGNED5.getLength(n2) - length);
            this.archiveSize0 += (long)(PackageWriter.UNSIGNED5.getLength(n3) - length);
        }
        if (this.verbose > 1) {
            Utils.log.fine("archive sizes: " + this.archiveSize0 + "+" + this.archiveSize1);
        }
        assert this.all_bands.outputSize() == this.archiveSize0 + this.archiveSize1;
        return;
    }
    
    void writeConstantPoolCounts(boolean b, boolean b2) throws IOException {
        for (int n : ConstantPool.TAGS_IN_ORDER) {
            int size = this.pkg.cp.getIndexByTag((byte)n).size();
            switch (n) {
                case 1: {
                    if (size > 0 && !PackageWriter.$assertionsDisabled && this.pkg.cp.getIndexByTag((byte)n).get(0) != ConstantPool.getUtf8Entry("")) {
                        throw new AssertionError();
                    }
                    break;
                }
                case 3:
                case 4:
                case 5:
                case 6: {
                    if (!b) {
                        assert size == 0;
                        continue;
                    }
                    else {
                        break;
                    }
                }
                case 15:
                case 16:
                case 17:
                case 18: {
                    if (!b2) {
                        assert size == 0;
                        continue;
                    }
                    else {
                        break;
                    }
                }
            }
            this.archive_header_1.putInt(size);
        }
    }
    
    @Override
    protected ConstantPool.Index getCPIndex(byte b) {
        return this.pkg.cp.getIndexByTag(b);
    }
    
    void writeConstantPool() throws IOException {
        ConstantPool.IndexGroup cp = this.pkg.cp;
        if (this.verbose > 0) {
            Utils.log.info("Writing CP");
        }
        for (int n : ConstantPool.TAGS_IN_ORDER) {
            ConstantPool.Index indexByTag = cp.getIndexByTag((byte)n);
            ConstantPool.Entry[] cpMap = indexByTag.cpMap;
            if (this.verbose > 0) {
                Utils.log.info("Writing " + cpMap.length + " " + ConstantPool.tagName(n) + " entries...");
            }
            if (this.optDumpBands) {
                try (final Object o = new PrintStream(BandStructure.getDumpStream(indexByTag, ".idx"))) {
                    BandStructure.printArrayTo(o, cpMap, 0, cpMap.length);
                }
            }
            switch (n) {
                case 1: {
                    this.writeUtf8Bands(cpMap);
                    break;
                }
                case 3: {
                    for (int j = 0; j < cpMap.length; ++j) {
                        this.cp_Int.putInt((int)((ConstantPool.NumberEntry)cpMap[j]).numberValue());
                    }
                    break;
                }
                case 4: {
                    for (int k = 0; k < cpMap.length; ++k) {
                        this.cp_Float.putInt(Float.floatToIntBits((float)((ConstantPool.NumberEntry)cpMap[k]).numberValue()));
                    }
                    break;
                }
                case 5: {
                    for (int l = 0; l < cpMap.length; ++l) {
                        long longValue = (long)((ConstantPool.NumberEntry)cpMap[l]).numberValue();
                        this.cp_Long_hi.putInt((int)(longValue >>> 32));
                        this.cp_Long_lo.putInt((int)(longValue >>> 0));
                    }
                    break;
                }
                case 6: {
                    for (int n2 = 0; n2 < cpMap.length; ++n2) {
                        long doubleToLongBits = Double.doubleToLongBits((double)((ConstantPool.NumberEntry)cpMap[n2]).numberValue());
                        this.cp_Double_hi.putInt((int)(doubleToLongBits >>> 32));
                        this.cp_Double_lo.putInt((int)(doubleToLongBits >>> 0));
                    }
                    break;
                }
                case 8: {
                    for (int n3 = 0; n3 < cpMap.length; ++n3) {
                        this.cp_String.putRef(((ConstantPool.StringEntry)cpMap[n3]).ref);
                    }
                    break;
                }
                case 7: {
                    for (int n4 = 0; n4 < cpMap.length; ++n4) {
                        this.cp_Class.putRef(((ConstantPool.ClassEntry)cpMap[n4]).ref);
                    }
                    break;
                }
                case 13: {
                    this.writeSignatureBands(cpMap);
                    break;
                }
                case 12: {
                    for (int n5 = 0; n5 < cpMap.length; ++n5) {
                        ConstantPool.DescriptorEntry descriptorEntry = (ConstantPool.DescriptorEntry)cpMap[n5];
                        this.cp_Descr_name.putRef(descriptorEntry.nameRef);
                        this.cp_Descr_type.putRef(descriptorEntry.typeRef);
                    }
                    break;
                }
                case 9: {
                    this.writeMemberRefs((byte)n, cpMap, this.cp_Field_class, this.cp_Field_desc);
                    break;
                }
                case 10: {
                    this.writeMemberRefs((byte)n, cpMap, this.cp_Method_class, this.cp_Method_desc);
                    break;
                }
                case 11: {
                    this.writeMemberRefs((byte)n, cpMap, this.cp_Imethod_class, this.cp_Imethod_desc);
                    break;
                }
                case 15: {
                    for (int n6 = 0; n6 < cpMap.length; ++n6) {
                        ConstantPool.MethodHandleEntry methodHandleEntry = (ConstantPool.MethodHandleEntry)cpMap[n6];
                        this.cp_MethodHandle_refkind.putInt(methodHandleEntry.refKind);
                        this.cp_MethodHandle_member.putRef(methodHandleEntry.memRef);
                    }
                    break;
                }
                case 16: {
                    for (int n7 = 0; n7 < cpMap.length; ++n7) {
                        this.cp_MethodType.putRef(((ConstantPool.MethodTypeEntry)cpMap[n7]).typeRef);
                    }
                    break;
                }
                case 18: {
                    for (int n8 = 0; n8 < cpMap.length; ++n8) {
                        ConstantPool.InvokeDynamicEntry invokeDynamicEntry = (ConstantPool.InvokeDynamicEntry)cpMap[n8];
                        this.cp_InvokeDynamic_spec.putRef(invokeDynamicEntry.bssRef);
                        this.cp_InvokeDynamic_desc.putRef(invokeDynamicEntry.descRef);
                    }
                    break;
                }
                case 17: {
                    for (int n9 = 0; n9 < cpMap.length; ++n9) {
                        ConstantPool.BootstrapMethodEntry bootstrapMethodEntry = (ConstantPool.BootstrapMethodEntry)cpMap[n9];
                        this.cp_BootstrapMethod_ref.putRef(bootstrapMethodEntry.bsmRef);
                        this.cp_BootstrapMethod_arg_count.putInt(bootstrapMethodEntry.argRefs.length);
                        ConstantPool.Entry[] argRefs = bootstrapMethodEntry.argRefs;
                        for (int length2 = argRefs.length, n10 = 0; n10 < length2; ++n10) {
                            this.cp_BootstrapMethod_arg.putRef(argRefs[n10]);
                        }
                    }
                    break;
                }
                default: {
                    throw new AssertionError("unexpected CP tag in package");
                }
            }
        }
        if (this.optDumpBands || this.verbose > 1) {
            for (int n11 = 50; n11 < 54; n11 = (byte)(n11 + 1)) {
                ConstantPool.Index indexByTag2 = cp.getIndexByTag((byte)n11);
                if (indexByTag2 != null && !indexByTag2.isEmpty()) {
                    ConstantPool.Entry[] cpMap2 = indexByTag2.cpMap;
                    if (this.verbose > 1) {
                        Utils.log.info("Index group " + ConstantPool.tagName(n11) + " contains " + cpMap2.length + " entries.");
                    }
                    if (this.optDumpBands) {
                        try (final Object o2 = new PrintStream(BandStructure.getDumpStream(indexByTag2.debugName, n11, ".gidx", indexByTag2))) {
                            BandStructure.printArrayTo(o2, cpMap2, 0, cpMap2.length, true);
                        }
                    }
                }
            }
        }
    }
    
    void writeUtf8Bands(ConstantPool.Entry[] array) throws IOException {
        if (array.length == 0) {
            return;
        }
        assert !(!array[0].stringValue().equals(""));
        char[][] array2 = new char[array.length][];
        for (int i = 0; i < array2.length; ++i) {
            array2[i] = array[i].stringValue().toCharArray();
        }
        int[] array3 = new int[array.length];
        char[] array4 = new char[0];
        char[] array5;
        for (int j = 0; j < array2.length; array4 = array5, ++j) {
            int n;
            for (n = 0, array5 = array2[j]; n < Math.min(array5.length, array4.length) && array5[n] == array4[n]; ++n) {}
            array3[j] = n;
            if (j >= 2) {
                this.cp_Utf8_prefix.putInt(n);
            }
            else
                assert n == 0;
        }
        for (int k = 0; k < array2.length; ++k) {
            char[] array6 = array2[k];
            int n2 = array3[k];
            int n3 = array6.length - array3[k];
            boolean tryAlternateEncoding = false;
            if (n3 == 0) {
                tryAlternateEncoding = (k >= 1);
            }
            else if (this.optBigStrings && this.effort > 1 && n3 > 100) {
                int n4 = 0;
                for (int l = 0; l < n3; ++l) {
                    if (array6[n2 + l] > '\u007f') {
                        ++n4;
                    }
                }
                if (n4 > 100) {
                    tryAlternateEncoding = this.tryAlternateEncoding(k, n4, array6, n2);
                }
            }
            if (k < 1) {
                assert tryAlternateEncoding == 0;
                assert n3 == 0;
            }
            else if (tryAlternateEncoding != 0) {
                this.cp_Utf8_suffix.putInt(0);
                this.cp_Utf8_big_suffix.putInt(n3);
            }
            else {
                assert n3 != 0;
                this.cp_Utf8_suffix.putInt(n3);
                for (int n5 = 0; n5 < n3; ++n5) {
                    this.cp_Utf8_chars.putInt((int)array6[n2 + n5]);
                }
            }
        }
        if (this.verbose > 0) {
            int length = this.cp_Utf8_chars.length();
            int length2 = this.cp_Utf8_big_chars.length();
            Utils.log.info("Utf8string #CHARS=" + (length + length2) + " #PACKEDCHARS=" + length2);
        }
        return;
    }
    
    private boolean tryAlternateEncoding(int i, int j, char[] array, int n) {
        int k = array.length - n;
        int[] array2 = new int[k];
        for (int l = 0; l < k; ++l) {
            array2[l] = array[n + l];
        }
        CodingChooser codingChooser = this.getCodingChooser();
        Coding regularCoding = this.cp_Utf8_big_chars.regularCoding;
        String string = "(Utf8_big_" + i + ")";
        int[] array3 = { 0, 0 };
        if (this.verbose > 1 || codingChooser.verbose > 1) {
            Utils.log.fine("--- chooseCoding " + string);
        }
        CodingMethod choose = codingChooser.choose(array2, regularCoding, array3);
        Coding regularCoding2 = this.cp_Utf8_chars.regularCoding;
        if (this.verbose > 1) {
            Utils.log.fine("big string[" + i + "] len=" + k + " #wide=" + j + " size=" + array3[0] + "/z=" + array3[1] + " coding " + choose);
        }
        if (choose != regularCoding2) {
            int n2 = array3[1];
            int[] computeSize = codingChooser.computeSize(regularCoding2, array2);
            int n3 = computeSize[1];
            int max = Math.max(5, n3 / 1000);
            if (this.verbose > 1) {
                Utils.log.fine("big string[" + i + "] normalSize=" + computeSize[0] + "/z=" + computeSize[1] + " win=" + (n2 < n3 - max));
            }
            if (n2 < n3 - max) {
                this.cp_Utf8_big_chars.newIntBand(string).initializeValues(array2);
                return true;
            }
        }
        return false;
    }
    
    void writeSignatureBands(ConstantPool.Entry[] array) throws IOException {
        for (int i = 0; i < array.length; ++i) {
            ConstantPool.SignatureEntry signatureEntry = (ConstantPool.SignatureEntry)array[i];
            this.cp_Signature_form.putRef(signatureEntry.formRef);
            for (int j = 0; j < signatureEntry.classRefs.length; ++j) {
                this.cp_Signature_classes.putRef(signatureEntry.classRefs[j]);
            }
        }
    }
    
    void writeMemberRefs(byte b, ConstantPool.Entry[] array, BandStructure.CPRefBand cpRefBand, BandStructure.CPRefBand cpRefBand2) throws IOException {
        for (int i = 0; i < array.length; ++i) {
            ConstantPool.MemberEntry memberEntry = (ConstantPool.MemberEntry)array[i];
            cpRefBand.putRef(memberEntry.classRef);
            cpRefBand2.putRef(memberEntry.descRef);
        }
    }
    
    void writeFiles() throws IOException {
        int size = this.pkg.files.size();
        if (size == 0) {
            return;
        }
        int archiveOptions = this.archiveOptions;
        int testBit = BandStructure.testBit(archiveOptions, 256);
        int testBit2 = BandStructure.testBit(archiveOptions, 64);
        int testBit3 = BandStructure.testBit(archiveOptions, 128);
        if (testBit3 == 0) {
            Iterator<Package.File> iterator = this.pkg.files.iterator();
            while (iterator.hasNext()) {
                if (((Package.File)iterator.next()).isClassStub()) {
                    testBit3 = 1;
                    archiveOptions |= 0x80;
                    this.archiveOptions = archiveOptions;
                    break;
                }
                continue;
            }
        }
        if (testBit || testBit2 || testBit3 != 0 || !this.pkg.files.isEmpty()) {
            this.archiveOptions = (archiveOptions | 0x10);
        }
        for (Package.File file : this.pkg.files) {
            this.file_name.putRef(file.name);
            long fileLength = file.getFileLength();
            this.file_size_lo.putInt((int)fileLength);
            if (!(testBit == 0)) {
                this.file_size_hi.putInt((int)(fileLength >>> 32));
            }
            if (!(testBit2 == 0)) {
                this.file_modtime.putInt(file.modtime - this.pkg.default_modtime);
            }
            if (testBit3 != 0) {
                this.file_options.putInt(file.options);
            }
            file.writeTo(this.file_bits.collectorStream());
            if (this.verbose > 1) {
                Utils.log.fine("Wrote " + fileLength + " bytes of " + file.name.stringValue());
            }
            else {
                continue;
            }
        }
        if (this.verbose > 0) {
            Utils.log.info("Wrote " + size + " resource files");
        }
        return;
    }
    
    void collectAttributeLayouts() {
        this.maxFlags = new int[4];
        this.allLayouts = new FixedList(4);
        for (int i = 0; i < 4; ++i) {
            this.allLayouts.set(i, new HashMap());
        }
        for (Package.Class class1 : this.pkg.classes) {
            this.visitAttributeLayoutsIn(0, class1);
            Iterator<Package.Class.Field> iterator2 = class1.getFields().iterator();
            while (iterator2.hasNext()) {
                this.visitAttributeLayoutsIn(1, (Package.Class.Field)iterator2.next());
            }
            for (Package.Class.Method method : class1.getMethods()) {
                this.visitAttributeLayoutsIn(2, method);
                if (method.code != null) {
                    this.visitAttributeLayoutsIn(3, method.code);
                }
                else {
                    continue;
                }
            }
        }
        int j = 0;
        while (j < 4) {
            int size = ((Map)this.allLayouts.get(j)).size();
            int haveFlagsHi = this.haveFlagsHi(j);
            if (size >= 24) {
                this.archiveOptions |= 1 << 9 + j;
                haveFlagsHi = true;
                if (this.verbose > 0) {
                    Utils.log.info("Note: Many " + Attribute.contextName(j) + " attributes forces 63-bit flags");
                }
            }
            if (this.verbose > 1) {
                Utils.log.fine(Attribute.contextName(j) + ".maxFlags = 0x" + Integer.toHexString(this.maxFlags[j]));
                Utils.log.fine(Attribute.contextName(j) + ".#layouts = " + size);
            }
            assert (this.haveFlagsHi(j) ? 1 : 0) == haveFlagsHi;
            ++j;
        }
        this.initAttrIndexLimit();
        int k = 0;
        while (k < 4) {
            assert (this.attrFlagMask[k] & (long)this.maxFlags[k]) == 0x0L;
            ++k;
        }
        this.backCountTable = new HashMap();
        this.attrCounts = new int[4][];
        int l = 0;
        while (l < 4) {
            long n = ~((long)this.maxFlags[l] | this.attrFlagMask[l]);
            assert this.attrIndexLimit[l] > 0;
            assert this.attrIndexLimit[l] < 64;
            long n2 = n & (1L << this.attrIndexLimit[l]) - 1L;
            int n3 = 0;
            Map<Object, Object> map = (Map)this.allLayouts.get(l);
            Map.Entry[] a = new Map.Entry[map.size()];
            map.entrySet().toArray(a);
            Arrays.sort(a, new Comparator<Map.Entry<Attribute.Layout, int[]>>() {
                @Override
                public int compare(Map.Entry<Attribute.Layout, int[]> entry, Map.Entry<Attribute.Layout, int[]> entry2) {
                    int n = -(((int[])entry.getValue())[0] - ((int[])entry2.getValue())[0]);
                    if (n != 0) {
                        return n;
                    }
                    return ((Attribute.Layout)entry.getKey()).compareTo((Attribute.Layout)entry2.getKey());
                }
            });
            this.attrCounts[l] = new int[this.attrIndexLimit[l] + a.length];
            for (int n4 = 0; n4 < a.length; ++n4) {
                Map.Entry<Object, Object> entry = a[n4];
                Attribute.Layout layout = (Attribute.Layout)entry.getKey();
                int n5 = ((int[])entry.getValue())[0];
                Integer n6 = (Integer)this.attrIndexTable.get(layout);
                int n7;
                if (n6 != null) {
                    n7 = (int)n6;
                }
                else if (n2 != 0L) {
                    while ((n2 & 0x1L) == 0x0L) {
                        n2 >>>= 1;
                        ++n3;
                    }
                    --n2;
                    n7 = this.setAttributeLayoutIndex(layout, n3);
                }
                else {
                    n7 = this.setAttributeLayoutIndex(layout, -1);
                }
                this.attrCounts[l][n7] = n5;
                Attribute.Layout.Element[] callables = layout.getCallables();
                int[] array = new int[callables.length];
                int n8 = 0;
                while (n8 < callables.length) {
                    assert callables[n8].kind == 10;
                    if (!callables[n8].flagTest((byte)8)) {
                        array[n8] = -1;
                    }
                    ++n8;
                }
                this.backCountTable.put(layout, array);
                if (n6 == null) {
                    ConstantPool.Utf8Entry utf8Entry = ConstantPool.getUtf8Entry(layout.name());
                    ConstantPool.Utf8Entry utf8Entry2 = ConstantPool.getUtf8Entry(layout.layoutForClassVersion(this.getHighestClassVersion()));
                    this.requiredEntries.add(utf8Entry);
                    this.requiredEntries.add(utf8Entry2);
                    if (this.verbose > 0) {
                        if (n7 < this.attrIndexLimit[l]) {
                            Utils.log.info("Using free flag bit 1<<" + n7 + " for " + n5 + " occurrences of " + layout);
                        }
                        else {
                            Utils.log.info("Using overflow index " + n7 + " for " + n5 + " occurrences of " + layout);
                        }
                    }
                }
            }
            ++l;
        }
        this.maxFlags = null;
        this.allLayouts = null;
    }
    
    void visitAttributeLayoutsIn(int n, Attribute.Holder holder) {
        int[] maxFlags = this.maxFlags;
        maxFlags[n] |= holder.flags;
        Iterator<Attribute> iterator = holder.getAttributes().iterator();
        while (iterator.hasNext()) {
            Attribute.Layout layout = ((Attribute)iterator.next()).layout();
            Map<Attribute.Layout, Object> map = (Map)this.allLayouts.get(n);
            int[] array = (int[])map.get(layout);
            if (array == null) {
                map.put(layout, array = new int[] { 0 });
            }
            if (array[0] < Integer.MAX_VALUE) {
                int n2 = 0;
                ++array[n2];
            }
            else {
                continue;
            }
        }
    }
    
    void writeAttrDefs() throws IOException {
        ArrayList<Object> list = new ArrayList();
        for (int i = 0; i < 4; ++i) {
            for (int size = ((List)this.attrDefs.get(i)).size(), j = 0; j < size; ++j) {
                if (j < this.attrIndexLimit[i]) {
                    i |= j + 1 << 2;
                    assert i < 256;
                    if (!BandStructure.testBit(this.attrDefSeen[i], 1L << j)) {
                        continue;
                    }
                }
                Attribute.Layout layout = (Attribute.Layout)((List)this.attrDefs.get(i)).get(j);
                list.add(new Object[] { Integer.valueOf(i), layout });
                assert !(!Integer.valueOf(j).equals(this.attrIndexTable.get(layout)));
            }
        }
        int size2 = list.size();
        Object[][] a = new Object[size2][];
        list.toArray(a);
        Arrays.sort(a, new Comparator<Object[]>(this) {
            @Override
            public int compare(Object[] array, Object[] array2) {
                int compareTo = ((Comparable)array[0]).compareTo(array2[0]);
                if (compareTo != 0) {
                    return compareTo;
                }
                Integer n = (Integer)attrIndexTable.get(array[1]);
                Integer anotherInteger = (Integer)attrIndexTable.get(array2[1]);
                assert n != null;
                assert anotherInteger != null;
                return n.compareTo(anotherInteger);
            }
        });
        this.attrDefsWritten = new Attribute.Layout[size2];
        try (final Object o = this.optDumpBands ? new PrintStream(BandStructure.getDumpStream(this.attr_definition_headers, ".def")) : null) {
            int[] copy = Arrays.copyOf(this.attrIndexLimit, 4);
            int k = 0;
            while (k < a.length) {
                int intValue = (int)a[k][0];
                Attribute.Layout obj = (Attribute.Layout)a[k][1];
                this.attrDefsWritten[k] = obj;
                assert (intValue & 0x3) == obj.ctype();
                this.attr_definition_headers.putByte(intValue);
                this.attr_definition_name.putRef(ConstantPool.getUtf8Entry(obj.name()));
                this.attr_definition_layout.putRef(ConstantPool.getUtf8Entry(obj.layoutForClassVersion(this.getHighestClassVersion())));
                boolean n = false;
                assert !((n = true) == 0);
                if (n != 0) {
                    int n2 = (intValue >> 2) - 1;
                    if (n2 < 0) {
                        n2 = copy[obj.ctype()]++;
                    }
                    int intValue2 = (int)this.attrIndexTable.get(obj);
                    assert n2 == intValue2;
                }
                if (o != null) {
                    o.println((intValue >> 2) - 1 + " " + obj);
                }
                ++k;
            }
        }
    }
    
    void writeAttrCounts() throws IOException {
        for (int i = 0; i < 4; ++i) {
            BandStructure.IntBand attrBand = BandStructure.getAttrBand(this.attrBands[i], 4);
            Attribute.Layout[] array = new Attribute.Layout[((List)this.attrDefs.get(i)).size()];
            ((List)this.attrDefs.get(i)).toArray(array);
            boolean n = true;
            while (true) {
                for (int j = 0; j < array.length; ++j) {
                    Attribute.Layout layout = array[j];
                    if (layout != null && n == (this.isPredefinedAttr(i, j) ? 1 : 0) && this.attrCounts[i][j] != 0) {
                        int[] array2 = (int[])this.backCountTable.get(layout);
                        for (int k = 0; k < array2.length; ++k) {
                            if (array2[k] >= 0) {
                                int n2 = array2[k];
                                array2[k] = -1;
                                attrBand.putInt(n2);
                                assert !(!layout.getCallables()[k].flagTest((byte)8));
                            }
                            else
                                assert !layout.getCallables()[k].flagTest((byte)8);
                        }
                    }
                }
                if (n == 0) {
                    break;
                }
                n = false;
            }
        }
    }
    
    void trimClassAttributes() {
        for (Package.Class class1 : this.pkg.classes) {
            class1.minimizeSourceFile();
            assert class1.getAttribute(Package.attrBootstrapMethodsEmpty) == null;
            continue;
        }
    }
    
    void collectInnerClasses() {
        HashMap<Package.InnerClass, Package.InnerClass> hashMap = new HashMap();
        for (Package.Class class1 : this.pkg.classes) {
            if (!class1.hasInnerClasses()) {
                continue;
            }
            for (Package.InnerClass innerClass : class1.getInnerClasses()) {
                Package.InnerClass innerClass2 = (Package.InnerClass)hashMap.put(innerClass.thisClass, innerClass);
                if (innerClass2 != null && !innerClass2.equals(innerClass) && innerClass2.predictable) {
                    hashMap.put(innerClass2.thisClass, innerClass2);
                }
                else {
                    continue;
                }
            }
        }
        Package.InnerClass[] array = new Package.InnerClass[hashMap.size()];
        hashMap.values().toArray(array);
        Arrays.sort(array);
        this.pkg.setAllInnerClasses(Arrays.asList(array));
        Iterator<Package.Class> iterator3 = this.pkg.classes.iterator();
        while (iterator3.hasNext()) {
            ((Package.Class)iterator3.next()).minimizeLocalICs();
        }
    }
    
    void writeInnerClasses() throws IOException {
        for (Package.InnerClass innerClass : this.pkg.getAllInnerClasses()) {
            int flags = innerClass.flags;
            assert (flags & 0x10000) == 0x0;
            if (!innerClass.predictable) {
                flags |= 0x10000;
            }
            this.ic_this_class.putRef(innerClass.thisClass);
            this.ic_flags.putInt(flags);
            if (!innerClass.predictable) {
                this.ic_outer_class.putRef(innerClass.outerClass);
                this.ic_name.putRef(innerClass.name);
            }
            else {
                continue;
            }
        }
    }
    
    void writeLocalInnerClasses(Package.Class class1) throws IOException {
        List<Package.InnerClass> innerClasses = class1.getInnerClasses();
        this.class_InnerClasses_N.putInt(innerClasses.size());
        for (Package.InnerClass innerClass : innerClasses) {
            this.class_InnerClasses_RC.putRef(innerClass.thisClass);
            if (innerClass.equals(this.pkg.getGlobalInnerClass(innerClass.thisClass))) {
                this.class_InnerClasses_F.putInt(0);
            }
            else {
                int flags = innerClass.flags;
                if (flags == 0) {
                    flags = 65536;
                }
                this.class_InnerClasses_F.putInt(flags);
                this.class_InnerClasses_outer_RCN.putRef(innerClass.outerClass);
                this.class_InnerClasses_name_RUN.putRef(innerClass.name);
            }
        }
    }
    
    void writeClassesAndByteCodes() throws IOException {
        Package.Class[] a = new Package.Class[this.pkg.classes.size()];
        this.pkg.classes.toArray(a);
        if (this.verbose > 0) {
            Utils.log.info("  ...scanning " + a.length + " classes...");
        }
        int i = 0;
        int j = 0;
        while (j < a.length) {
            Package.Class obj = a[j];
            if (this.verbose > 1) {
                Utils.log.fine("Scanning " + obj);
            }
            ConstantPool.ClassEntry thisClass = obj.thisClass;
            ConstantPool.ClassEntry superClass = obj.superClass;
            ConstantPool.ClassEntry[] interfaces = obj.interfaces;
            assert superClass != thisClass;
            if (superClass == null) {
                superClass = thisClass;
            }
            this.class_this.putRef(thisClass);
            this.class_super.putRef(superClass);
            this.class_interface_count.putInt(obj.interfaces.length);
            for (int k = 0; k < interfaces.length; ++k) {
                this.class_interface.putRef(interfaces[k]);
            }
            this.writeMembers(obj);
            this.writeAttrs(0, obj, obj);
            ++i;
            if (this.verbose > 0 && i % 1000 == 0) {
                Utils.log.info("Have scanned " + i + " classes...");
            }
            ++j;
        }
    }
    
    void writeMembers(Package.Class class1) throws IOException {
        List<Package.Class.Field> fields = class1.getFields();
        this.class_field_count.putInt(fields.size());
        for (Package.Class.Field field : fields) {
            this.field_descr.putRef(field.getDescriptor());
            this.writeAttrs(1, field, class1);
        }
        List<Package.Class.Method> methods = class1.getMethods();
        this.class_method_count.putInt(methods.size());
        for (Package.Class.Method method : methods) {
            this.method_descr.putRef(method.getDescriptor());
            this.writeAttrs(2, method, class1);
            assert method.code != null == (method.getAttribute(this.attrCodeEmpty) != null);
            if (method.code != null) {
                this.writeCodeHeader(method.code);
                this.writeByteCodes(method.code);
            }
            else {
                continue;
            }
        }
    }
    
    void writeCodeHeader(Code code) throws IOException {
        int testBit = BandStructure.testBit(this.archiveOptions, 4);
        int attributeSize = code.attributeSize();
        int shortCodeHeader = BandStructure.shortCodeHeader(code);
        if (testBit == 0 && attributeSize > 0) {
            shortCodeHeader = 0;
        }
        if (this.verbose > 2) {
            Utils.log.fine("Code sizes info " + code.max_stack + " " + code.max_locals + " " + code.getHandlerCount() + " " + code.getMethod().getArgumentSize() + " " + attributeSize + ((shortCodeHeader > 0) ? (" SHORT=" + shortCodeHeader) : ""));
        }
        this.code_headers.putByte(shortCodeHeader);
        if (shortCodeHeader == 0) {
            this.code_max_stack.putInt(code.getMaxStack());
            this.code_max_na_locals.putInt(code.getMaxNALocals());
            this.code_handler_count.putInt(code.getHandlerCount());
        }
        else {
            assert attributeSize == 0;
            assert code.getHandlerCount() < this.shortCodeHeader_h_limit;
        }
        this.writeCodeHandlers(code);
        if (shortCodeHeader == 0 || testBit) {
            this.writeAttrs(3, code, code.thisClass());
        }
    }
    
    void writeCodeHandlers(Code code) throws IOException {
        for (int i = 0; i < code.getHandlerCount(); ++i) {
            this.code_handler_class_RCN.putRef(code.handler_class[i]);
            int encodeBCI = code.encodeBCI(code.handler_start[i]);
            this.code_handler_start_P.putInt(encodeBCI);
            int n = code.encodeBCI(code.handler_end[i]) - encodeBCI;
            this.code_handler_end_PO.putInt(n);
            this.code_handler_catch_PO.putInt(code.encodeBCI(code.handler_catch[i]) - (encodeBCI + n));
        }
    }
    
    void writeAttrs(int n, final Attribute.Holder holder, Package.Class class1) throws IOException {
        BandStructure.MultiBand multiBand = this.attrBands[n];
        BandStructure.IntBand attrBand = BandStructure.getAttrBand(multiBand, 0);
        BandStructure.IntBand attrBand2 = BandStructure.getAttrBand(multiBand, 1);
        int haveFlagsHi = this.haveFlagsHi(n);
        assert this.attrIndexLimit[n] == (haveFlagsHi ? 63 : 32);
        if (holder.attributes == null) {
            attrBand2.putInt(holder.flags);
            if (!(haveFlagsHi == 0)) {
                attrBand.putInt(0);
            }
            return;
        }
        else {
            if (this.verbose > 3) {
                Utils.log.fine("Transmitting attrs for " + holder + " flags=" + Integer.toHexString(holder.flags));
            }
            long i = this.attrFlagMask[n];
            long j = 0L;
            int k = 0;
            for (Attribute attribute : holder.attributes) {
                Attribute.Layout layout = attribute.layout();
                int intValue = (int)this.attrIndexTable.get(layout);
                assert ((List)this.attrDefs.get(n)).get(intValue) == layout;
                if (this.verbose > 3) {
                    Utils.log.fine("add attr @" + intValue + " " + attribute + " in " + holder);
                }
                if (intValue < this.attrIndexLimit[n] && BandStructure.testBit(i, 1L << intValue)) {
                    if (this.verbose > 3) {
                        Utils.log.fine("Adding flag bit 1<<" + intValue + " in " + Long.toHexString(i));
                    }
                    assert !BandStructure.testBit((long)holder.flags, 1L << intValue);
                    j |= 1L << intValue;
                    i -= 1L << intValue;
                }
                else {
                    j |= 0x10000L;
                    ++k;
                    if (this.verbose > 3) {
                        Utils.log.fine("Adding overflow attr #" + k);
                    }
                    BandStructure.getAttrBand(multiBand, 3).putInt(intValue);
                }
                if (layout.bandCount == 0) {
                    if (layout != this.attrInnerClassesEmpty) {
                        continue;
                    }
                    this.writeLocalInnerClasses((Package.Class)holder);
                }
                else {
                    assert attribute.fixups == null;
                    final BandStructure.Band[] array = (BandStructure.Band[])this.attrBandTable.get(layout);
                    assert array != null;
                    assert array.length == layout.bandCount;
                    final int[] array2 = (int[])this.backCountTable.get(layout);
                    assert array2 != null;
                    assert array2.length == layout.getCallables().length;
                    if (this.verbose > 2) {
                        Utils.log.fine("writing " + attribute + " in " + holder);
                    }
                    int n2 = n == 1 && layout == this.attrConstantValue;
                    if (!(n2 == 0)) {
                        this.setConstantValueIndex((Package.Class.Field)holder);
                    }
                    attribute.parse(class1, attribute.bytes(), 0, attribute.size(), new Attribute.ValueStream() {
                        @Override
                        public void putInt(int n, int n2) {
                            ((BandStructure.IntBand)array[n]).putInt(n2);
                        }
                        
                        @Override
                        public void putRef(int n, ConstantPool.Entry entry) {
                            ((BandStructure.CPRefBand)array[n]).putRef(entry);
                        }
                        
                        @Override
                        public int encodeBCI(int n) {
                            return ((Code)holder).encodeBCI(n);
                        }
                        
                        @Override
                        public void noteBackCall(int n) {
                            assert array2[n] >= 0;
                            int[] val$bc = array2;
                            ++bc[n];
                            return;
                        }
                    });
                    if (!(n2 == 0)) {
                        this.setConstantValueIndex(null);
                    }
                    else {
                        continue;
                    }
                }
            }
            if (k > 0) {
                BandStructure.getAttrBand(multiBand, 2).putInt(k);
            }
            attrBand2.putInt(holder.flags | (int)j);
            if (!(haveFlagsHi == 0)) {
                attrBand.putInt((int)(j >>> 32));
            }
            else
                assert j >>> 32 == 0L;
            assert ((long)holder.flags & j) == 0x0L : holder + ".flags=" + Integer.toHexString(holder.flags) + "^" + Long.toHexString(j);
            return;
        }
    }
    
    private void beginCode(Code curCode) {
        assert this.curCode == null;
        this.curCode = curCode;
        this.curClass = curCode.m.thisClass();
        this.curCPMap = curCode.getCPMap();
        return;
    }
    
    private void endCode() {
        this.curCode = null;
        this.curClass = null;
        this.curCPMap = null;
    }
    
    private int initOpVariant(Instruction instruction, ConstantPool.Entry entry) {
        if (instruction.getBC() != 183) {
            return -1;
        }
        ConstantPool.MemberEntry memberEntry = (ConstantPool.MemberEntry)instruction.getCPRef(this.curCPMap);
        if (!"<init>".equals(memberEntry.descRef.nameRef.stringValue())) {
            return -1;
        }
        ConstantPool.ClassEntry classRef = memberEntry.classRef;
        if (classRef == this.curClass.thisClass) {
            return 230;
        }
        if (classRef == this.curClass.superClass) {
            return 231;
        }
        if (classRef == entry) {
            return 232;
        }
        return -1;
    }
    
    private int selfOpVariant(Instruction instruction) {
        int bc = instruction.getBC();
        if (bc < 178 || bc > 184) {
            return -1;
        }
        ConstantPool.MemberEntry memberEntry = (ConstantPool.MemberEntry)instruction.getCPRef(this.curCPMap);
        if ((bc == 183 || bc == 184) && memberEntry.tagEquals(11)) {
            return -1;
        }
        ConstantPool.ClassEntry classRef = memberEntry.classRef;
        int n = 202 + (bc - 178);
        if (classRef == this.curClass.thisClass) {
            return n;
        }
        if (classRef == this.curClass.superClass) {
            return n + 14;
        }
        return -1;
    }
    
    void writeByteCodes(Code obj) throws IOException {
        this.beginCode(obj);
        ConstantPool.IndexGroup cp = this.pkg.cp;
        int n = 0;
        ConstantPool.Entry entry = null;
        Instruction instruction = obj.instructionAt(0);
        while (instruction != null) {
            if (this.verbose > 3) {
                Utils.log.fine(instruction.toString());
            }
            if (instruction.isNonstandard()) {
                String string2 = obj.getMethod() + " contains an unrecognized bytecode " + instruction + "; please use the pass-file option on this class.";
                Utils.log.warning(string2);
                throw new IOException(string2);
            }
            if (instruction.isWide()) {
                if (this.verbose > 1) {
                    Utils.log.fine("_wide opcode in " + obj);
                    Utils.log.fine(instruction.toString());
                }
                this.bc_codes.putByte(196);
                int[] codeHist = this.codeHist;
                int n2 = 196;
                ++codeHist[n2];
            }
            int bc = instruction.getBC();
            if (bc == 42 && this.selfOpVariant(obj.instructionAt(instruction.getNextPC())) >= 0) {
                n = 1;
            }
            else {
                int initOpVariant = this.initOpVariant(instruction, entry);
                if (initOpVariant >= 0) {
                    if (n != 0) {
                        this.bc_codes.putByte(42);
                        int[] codeHist4 = this.codeHist;
                        int n6 = 42;
                        ++codeHist4[n6];
                        n = 0;
                    }
                    this.bc_codes.putByte(initOpVariant);
                    int[] codeHist5 = this.codeHist;
                    int n7 = initOpVariant;
                    ++codeHist5[n7];
                    this.bc_initref.putInt(cp.getOverloadingIndex((ConstantPool.MemberEntry)instruction.getCPRef(this.curCPMap)));
                }
                else {
                    int selfOpVariant = this.selfOpVariant(instruction);
                    if (selfOpVariant >= 0) {
                        Instruction.isFieldOp(bc);
                        int n4 = selfOpVariant >= 216;
                        n = 0;
                        if (n != 0) {
                            selfOpVariant += 7;
                        }
                        this.bc_codes.putByte(selfOpVariant);
                        int[] codeHist3 = this.codeHist;
                        int n5 = selfOpVariant;
                        ++codeHist3[n5];
                        ConstantPool.MemberEntry memberEntry = (ConstantPool.MemberEntry)instruction.getCPRef(this.curCPMap);
                        this.selfOpRefBand(selfOpVariant).putRef(memberEntry, cp.getMemberIndex(memberEntry.tag, memberEntry.classRef));
                    }
                    else {
                        assert n == 0;
                        int[] codeHist2 = this.codeHist;
                        int n3 = bc;
                        ++codeHist2[n3];
                        switch (bc) {
                            case 170:
                            case 171: {
                                this.bc_codes.putByte(bc);
                                Instruction.Switch switch1 = (Instruction.Switch)instruction;
                                switch1.getAlignedPC();
                                switch1.getNextPC();
                                int caseCount = switch1.getCaseCount();
                                this.bc_case_count.putInt(caseCount);
                                this.putLabel(this.bc_label, obj, instruction.getPC(), switch1.getDefaultLabel());
                                for (int i = 0; i < caseCount; ++i) {
                                    this.putLabel(this.bc_label, obj, instruction.getPC(), switch1.getCaseLabel(i));
                                }
                                if (bc == 170) {
                                    this.bc_case_value.putInt(switch1.getCaseValue(0));
                                    break;
                                }
                                for (int j = 0; j < caseCount; ++j) {
                                    this.bc_case_value.putInt(switch1.getCaseValue(j));
                                }
                                break;
                            }
                            default: {
                                int branchLabel = instruction.getBranchLabel();
                                if (branchLabel >= 0) {
                                    this.bc_codes.putByte(bc);
                                    this.putLabel(this.bc_label, obj, instruction.getPC(), branchLabel);
                                    break;
                                }
                                ConstantPool.Entry cpRef = instruction.getCPRef(this.curCPMap);
                                if (cpRef != null) {
                                    if (bc == 187) {
                                        entry = cpRef;
                                    }
                                    if (bc == 18) {
                                        int[] ldcHist = this.ldcHist;
                                        byte tag = cpRef.tag;
                                        ++ldcHist[tag];
                                    }
                                    BandStructure.CPRefBand cpRefBand;
                                    switch (instruction.getCPTag()) {
                                        case 51: {
                                            switch (cpRef.tag) {
                                                case 3: {
                                                    cpRefBand = this.bc_intref;
                                                    switch (bc) {
                                                        case 18: {
                                                            bc = 234;
                                                            break;
                                                        }
                                                        case 19: {
                                                            bc = 237;
                                                            break;
                                                        }
                                                        default: {
                                                            assert false;
                                                            break;
                                                        }
                                                    }
                                                    break;
                                                }
                                                case 4: {
                                                    cpRefBand = this.bc_floatref;
                                                    switch (bc) {
                                                        case 18: {
                                                            bc = 235;
                                                            break;
                                                        }
                                                        case 19: {
                                                            bc = 238;
                                                            break;
                                                        }
                                                        default: {
                                                            assert false;
                                                            break;
                                                        }
                                                    }
                                                    break;
                                                }
                                                case 5: {
                                                    cpRefBand = this.bc_longref;
                                                    assert bc == 20;
                                                    bc = 20;
                                                    break;
                                                }
                                                case 6: {
                                                    cpRefBand = this.bc_doubleref;
                                                    assert bc == 20;
                                                    bc = 239;
                                                    break;
                                                }
                                                case 8: {
                                                    cpRefBand = this.bc_stringref;
                                                    switch (bc) {
                                                        case 18: {
                                                            bc = 18;
                                                            break;
                                                        }
                                                        case 19: {
                                                            bc = 19;
                                                            break;
                                                        }
                                                        default: {
                                                            assert false;
                                                            break;
                                                        }
                                                    }
                                                    break;
                                                }
                                                case 7: {
                                                    cpRefBand = this.bc_classref;
                                                    switch (bc) {
                                                        case 18: {
                                                            bc = 233;
                                                            break;
                                                        }
                                                        case 19: {
                                                            bc = 236;
                                                            break;
                                                        }
                                                        default: {
                                                            assert false;
                                                            break;
                                                        }
                                                    }
                                                    break;
                                                }
                                                default: {
                                                    if (this.getHighestClassVersion().lessThan(Constants.JAVA7_MAX_CLASS_VERSION)) {
                                                        throw new IOException("bad class file major version for Java 7 ldc");
                                                    }
                                                    cpRefBand = this.bc_loadablevalueref;
                                                    switch (bc) {
                                                        case 18: {
                                                            bc = 240;
                                                            break;
                                                        }
                                                        case 19: {
                                                            bc = 241;
                                                            break;
                                                        }
                                                        default: {
                                                            assert false;
                                                            break;
                                                        }
                                                    }
                                                    break;
                                                }
                                            }
                                            break;
                                        }
                                        case 7: {
                                            if (cpRef == this.curClass.thisClass) {
                                                cpRef = null;
                                            }
                                            cpRefBand = this.bc_classref;
                                            break;
                                        }
                                        case 9: {
                                            cpRefBand = this.bc_fieldref;
                                            break;
                                        }
                                        case 10: {
                                            if (cpRef.tagEquals(11)) {
                                                if (bc == 183) {
                                                    bc = 242;
                                                }
                                                if (bc == 184) {
                                                    bc = 243;
                                                }
                                                cpRefBand = this.bc_imethodref;
                                                break;
                                            }
                                            else {
                                                cpRefBand = this.bc_methodref;
                                                break;
                                            }
                                        }
                                        case 11: {
                                            cpRefBand = this.bc_imethodref;
                                            break;
                                        }
                                        case 18: {
                                            cpRefBand = this.bc_indyref;
                                            break;
                                        }
                                        default: {
                                            cpRefBand = null;
                                            assert false;
                                            break;
                                        }
                                    }
                                    if (cpRef != null && cpRefBand.index != null && !cpRefBand.index.contains(cpRef)) {
                                        String string = obj.getMethod() + " contains a bytecode " + instruction + " with an unsupported constant reference; please use the pass-file option on this class.";
                                        Utils.log.warning(string);
                                        throw new IOException(string);
                                    }
                                    this.bc_codes.putByte(bc);
                                    cpRefBand.putRef(cpRef);
                                    if (bc == 197) {
                                        assert instruction.getConstant() == obj.getByte(instruction.getPC() + 3);
                                        this.bc_byte.putByte(0xFF & instruction.getConstant());
                                        break;
                                    }
                                    else if (bc == 185) {
                                        assert instruction.getLength() == 5;
                                        assert instruction.getConstant() == 1 + ((ConstantPool.MemberEntry)cpRef).descRef.typeRef.computeSize(true) << 8;
                                        break;
                                    }
                                    else if (bc == 186) {
                                        if (this.getHighestClassVersion().lessThan(Constants.JAVA7_MAX_CLASS_VERSION)) {
                                            throw new IOException("bad class major version for Java 7 invokedynamic");
                                        }
                                        assert instruction.getLength() == 5;
                                        assert instruction.getConstant() == 0;
                                        break;
                                    }
                                    else {
                                        assert instruction.getLength() == ((bc == 18) ? 2 : 3);
                                        break;
                                    }
                                }
                                else {
                                    int localSlot = instruction.getLocalSlot();
                                    if (localSlot >= 0) {
                                        this.bc_codes.putByte(bc);
                                        this.bc_local.putInt(localSlot);
                                        int constant = instruction.getConstant();
                                        if (bc == 132) {
                                            if (!instruction.isWide()) {
                                                this.bc_byte.putByte(0xFF & constant);
                                                break;
                                            }
                                            this.bc_short.putInt(0xFFFF & constant);
                                            break;
                                        }
                                        else {
                                            assert constant == 0;
                                            break;
                                        }
                                    }
                                    else {
                                        this.bc_codes.putByte(bc);
                                        if (instruction.getPC() + 1 < instruction.getNextPC()) {
                                            switch (bc) {
                                                case 17: {
                                                    this.bc_short.putInt(0xFFFF & instruction.getConstant());
                                                    break;
                                                }
                                                case 16: {
                                                    this.bc_byte.putByte(0xFF & instruction.getConstant());
                                                    break;
                                                }
                                                case 188: {
                                                    this.bc_byte.putByte(0xFF & instruction.getConstant());
                                                    break;
                                                }
                                                default: {
                                                    assert false;
                                                    break;
                                                }
                                            }
                                        }
                                        else {
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            instruction = instruction.next();
        }
        this.bc_codes.putByte(255);
        BandStructure.ByteBand bc_codes = this.bc_codes;
        ++bc_codes.elementCountForDebug;
        int[] codeHist6 = this.codeHist;
        int n8 = 255;
        ++codeHist6[n8];
        this.endCode();
    }
    
    void printCodeHist() {
        assert this.verbose > 0;
        String[] a = new String[this.codeHist.length];
        int i = 0;
        for (int j = 0; j < this.codeHist.length; ++j) {
            i += this.codeHist[j];
        }
        for (int k = 0; k < this.codeHist.length; ++k) {
            if (this.codeHist[k] == 0) {
                a[k] = "";
            }
            else {
                String byteName = Instruction.byteName(k);
                String string = "" + this.codeHist[k];
                String string2 = "         ".substring(string.length()) + string;
                String str;
                for (str = "" + this.codeHist[k] * 10000 / i; str.length() < 4; str = "0" + str) {}
                a[k] = string2 + "  " + (str.substring(0, str.length() - 2) + "." + str.substring(str.length() - 2)) + "%  " + byteName;
            }
        }
        Arrays.sort(a);
        System.out.println("Bytecode histogram [" + i + "]");
        int length = a.length;
        while (--length >= 0) {
            if ("".equals(a[length])) {
                continue;
            }
            System.out.println(a[length]);
        }
        for (int l = 0; l < this.ldcHist.length; ++l) {
            int m = this.ldcHist[l];
            if (m != 0) {
                System.out.println("ldc " + ConstantPool.tagName(l) + " " + m);
            }
        }
        return;
    }
}
